package org.artifactory.metrics.framework;

/**
 * Holds all feature adoption actions that are tracked as part of Feature Adoption.
 *
 * NOTE: Actions should be concise and usually one word. For example: finished, visited, created, opted_out.
 * For additional details, please use "dynamic labels"
 *
 * @author Oleksandr Legenkiy
 */
public enum FeatureAdoptionAction {

    CREATED("created"),
    UPDATED("updated"),
    VISITED("visited"),
    ARCHIVED("archived"),
    ADDED("added"),
    DOWNLOADED("downloaded"),

    // NOTE: all the actions from below should be replaced by actions from above. New actions can be added if needed,
    // but their names should follow the naming convention
    ARTIFACT_MIRRORED_TO_MEMBER("artifact_mirrored_to_member"),
    CREATED_NEW_FEDERATED_REPO("created_new_federated_repo"),
    ADDED_NEW_FEDERATED_MEMBER("added_new_federated_member"),
    UPDATED_FEDERATED_REPO("updated_federated_repo"),
    FULL_SYNC_TRIGGERED("full_sync_triggered"),
    TOKEN_REGISTRATION("token_registered"),
    PUBLISH_SYMBOLS_PACKAGE("publish_symbols_package"),
    DOWNLOAD_SYMBOL_FILE("download_symbol_file"),
    ;

    private final String action;

    FeatureAdoptionAction(String action) {
        this.action = action;
    }

    public String action() {
        return action;
    }
}
