package org.artifactory.security.access.emigrate;

import org.apache.commons.collections.CollectionUtils;
import org.artifactory.api.security.SecurityService;
import org.artifactory.security.access.AccessService;
import org.artifactory.security.access.emigrate.conveter.V5100ConvertResourceTypeToRepo;
import org.artifactory.security.access.emigrate.conveter.V600DecryptAllUsersCustomData;
import org.artifactory.security.access.emigrate.conveter.V6600CreateDefaultBuildAcl;
import org.jfrog.common.StreamSupportUtils;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author Omri Ziv
 */
public class AccessConvertersTest {

    @Mock
    private SecurityService securityService;
    @Mock
    private AccessService accessService;

    private AccessConverters accessConverters = null;
    private AutoCloseable autoCloseable;

    @BeforeClass
    public void init() {
        autoCloseable = MockitoAnnotations.openMocks(this);
        V5100ConvertResourceTypeToRepo resourceTypeConverter = new V5100ConvertResourceTypeToRepo(accessService);
        V600DecryptAllUsersCustomData decryptAllUsersCustomData = new V600DecryptAllUsersCustomData(securityService);
        V6600CreateDefaultBuildAcl createDefaultBuildAcl = new V6600CreateDefaultBuildAcl(accessService);

        accessConverters =
                new AccessConverters(resourceTypeConverter, decryptAllUsersCustomData, createDefaultBuildAcl);
    }

    @AfterClass
    public void tearDown() throws Exception {
        autoCloseable.close();
    }

    @Test
    public void testFindDuplicatesInGetAll() {
        Set<Class<? extends AccessConverter>> set = StreamSupportUtils.stream(accessConverters.getAll())
                .map(AccessConverter::getClass)
                .collect(Collectors.toSet());
        assertEquals(set.size(), accessConverters.getAll().size(), "AccessConverters.getAll contains duplication");
    }

    @Test
    public void testGetAllContainsAll() {
        Field[] fields = AccessConverters.class.getDeclaredFields();
        List<Class<? extends AccessConverter>> accessConverterClassList =
                StreamSupportUtils.stream(accessConverters.getAll())
                        .map(AccessConverter::getClass)
                        .collect(Collectors.toList());

        List<Class<? extends AccessConverter>> fieldClasses = Arrays.stream(fields)
                .map(Field::getType)
                .filter(AccessConverter.class::isAssignableFrom)
                .map(clazz -> (Class<? extends AccessConverter>) clazz)
                .collect(Collectors.toList());

        List<Class<? extends AccessConverter>> notContains = StreamSupportUtils.stream(fieldClasses)
                .filter(clazz -> !accessConverterClassList.contains(clazz))
                .collect(Collectors.toList());

        assertTrue(CollectionUtils.isEmpty(notContains),
                String.format("Converters %s not contains in AccessConverters.getAll() function", notContains));
    }
}
