package org.artifactory.concurrent;

import org.artifactory.common.ConstantValues;

import java.util.concurrent.TimeUnit;

import static org.artifactory.common.ConstantValues.federatedRepoFullSyncLockTimeMin;
import static org.artifactory.common.ConstantValues.federatedRepoFullSyncStaleLockTimeMin;
import static org.artifactory.common.ConstantValues.haHeartbeatStaleIntervalSecs;
import static org.artifactory.common.ConstantValues.lockLeasTime;
import static org.artifactory.concurrent.NonStandardLockCategory.Constants.CLUSTER_SINGLETON_LOCK_NAME;
import static org.artifactory.concurrent.NonStandardLockCategory.Constants.FED_REPO_FULL_SYNC_LOCK_NAME;
import static org.artifactory.concurrent.NonStandardLockCategory.Constants.FORCE_OPTIMIZATION_LOCK_NAME;
import static org.artifactory.concurrent.NonStandardLockCategory.Constants.REPLICATIONS_LOCK_NAME;

/**
 * Supports dynamic lock expiry & orphan lock criteria for non standard locks
 *
 * @author Uriah Levy
 * @author Mor Merhav
 */
public enum NonStandardLockCategory {


    CLUSTER_SINGLETON(CLUSTER_SINGLETON_LOCK_NAME, ConstantValues.haOrphanClusterSingletonLockPeriod.getInt()),
    REPLICATION(REPLICATIONS_LOCK_NAME, lockLeasTime.getInt() * 24,
            (int) TimeUnit.SECONDS.toMinutes(haHeartbeatStaleIntervalSecs.getLong() * 4L)),
    FORCE_BINARY_SHARDING_OPTIMIZATION(FORCE_OPTIMIZATION_LOCK_NAME, (int) TimeUnit.DAYS.toMinutes(1),
            (int) TimeUnit.DAYS.toMinutes(1)),
    FED_REPO_FULL_SYNC(FED_REPO_FULL_SYNC_LOCK_NAME, federatedRepoFullSyncLockTimeMin.getInt(),
            federatedRepoFullSyncStaleLockTimeMin.getInt());


    String name;
    int lockLeaseTimeMinutes;
    int orphanLockGraceTimeMinutes;

    NonStandardLockCategory(String name, int lockLeaseTimeMinutes, int orphanLockGraceTimeMinutes) {
        this.name = name;
        this.lockLeaseTimeMinutes = lockLeaseTimeMinutes;
        this.orphanLockGraceTimeMinutes = orphanLockGraceTimeMinutes;
    }

    NonStandardLockCategory(String name, int orphanLockGraceTimeMinutes) {
        this(name, -1, orphanLockGraceTimeMinutes);
    }

    public boolean isExpirable() {
        return lockLeaseTimeMinutes != -1;
    }

    public int getLockLeaseTimeMinutes() {
        return lockLeaseTimeMinutes;
    }

    public int getOrphanLockGraceTimeMinutes() {
        return orphanLockGraceTimeMinutes;
    }

    public String getName() {
        return name;
    }

    public abstract static class LockNames {


    }

    public static abstract class Constants {
        public static final String FED_REPO_FULL_SYNC_LOCK_NAME = "Full sync triggering";
        public static final String FORCE_OPTIMIZATION_LOCK_NAME = "force_optimization";
        public static final String REPLICATIONS_LOCK_NAME = "replications";
        public static final String CLUSTER_SINGLETON_LOCK_NAME = "cluster_singleton";
    }
}
