package org.artifactory.core.api.ph.model;

import com.google.common.collect.Maps;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import org.artifactory.core.api.ph.virtual.resolution.PackageRequestContext;

import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MultivaluedMap;
import java.util.List;
import java.util.Map;

@Immutable
@Getter
@EqualsAndHashCode
public class PackageDownloadContext implements PackageRequestContext {

    private final String path;
    private final String repoKey;
    @Nullable
    private final String alternativeUrl;
    private final Map<String, String> requestHeaders = Maps.newHashMap();

    public PackageDownloadContext(String repoKey, String path) {
        this(repoKey, path, null);
    }

    public PackageDownloadContext(String repoKey, String path, @Nullable String alternativeUrl) {
        this.repoKey = repoKey;
        this.path = path;
        this.alternativeUrl = alternativeUrl;
    }

    public PackageDownloadContext header(String key, String value) {
        if (key != null && value != null) {
            this.requestHeaders.put(key, value);
        }
        return this;
    }

    public PackageDownloadContext headers(HttpHeaders httpHeaders) {
        if (httpHeaders != null) {
            MultivaluedMap<String, String> multiMap = httpHeaders.getRequestHeaders();
            for (Map.Entry<String, List<String>> entry : multiMap.entrySet()) {
                List<String> requestHeader = entry.getValue();
                if (!requestHeader.isEmpty()) {
                    header(entry.getKey(), requestHeader.get(0));
                }
            }
        }
        return this;
    }

    public Map<String, String> getRequestHeaders() {
        return requestHeaders;
    }
}
