/*
 *
 * Copyright 2016 JFrog Ltd. All rights reserved.
 * JFROG PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package org.artifactory.core.api.ph.model;

import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.Maps;
import com.google.common.collect.SetMultimap;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.artifactory.core.api.ph.virtual.resolution.PackageUploadContext;

import javax.annotation.Nonnull;
import java.io.InputStream;
import java.util.Map;

/**
 * Upload context for package upload requests
 **/
@Getter
@Setter
@Accessors(chain = true)
public class PackageUploadContextImpl implements PackageUploadContext {

    private String repoKey;
    private final String path;
    private InputStream content;
    private long contentLength;
    private String sha1;
    private String sha256;
    private String md5;
    private final Map<String, String> requestHeaders = Maps.newHashMap();
    private SetMultimap<String, String> attributes = LinkedHashMultimap.create();

    public PackageUploadContextImpl(@Nonnull String repoKey, @Nonnull String path) {
        this.repoKey = repoKey;
        this.path = path;
    }

    public PackageUploadContextImpl header(String key, String value) {
        if (key != null && value != null) {
            this.requestHeaders.put(key, value);
        }
        return this;
    }

    public void addRequestHeaders(Map<String, String> requestHeaders) {
        this.requestHeaders.putAll(requestHeaders);
    }

    public PackageUploadContextImpl setAttributes(SetMultimap<String, String> attributes) {
        if (!attributes.isEmpty()) {
            this.attributes = attributes;
        }
        return this;
    }
}
