package org.artifactory.storage.db.aql.itest.service;

import org.artifactory.aql.model.AqlPermissionProvider;
import org.artifactory.storage.db.aql.parser.AqlParser;
import org.artifactory.storage.db.aql.parser.ParserElementResultContainer;
import org.artifactory.storage.db.aql.sql.builder.query.aql.AqlQuery;
import org.artifactory.storage.db.aql.sql.builder.query.aql.ParserToAqlAdapter;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;

/**
 * @author Tomer Slutsky
 */
public class AqlQueryValidatorTest extends ArtifactoryHomeBoundTest {
    private final AqlParser parser = new AqlParser();
    private final AqlQueryValidator validator = new AqlQueryValidator();
    private final ParserToAqlAdapter aqlAdapter = new ParserToAqlAdapter();

    @BeforeMethod
    public void before() {
        bindArtifactoryHome();
    }

    @AfterMethod
    public void after() {
        unbindArtifactoryHome();
    }

    @Test
    public void validateTransitiveQuery() {
        this.assertValid("items.find({\"repo\": \"works\"}).transitive()");
        this.assertValid("items.find({\"repo\": \"works\"}).include(\"repo\").transitive()");
        this.assertValid("items.find({\"repo\": \"works\"},{\"name\":\"working\"}).include(\"repo\").transitive()");
        this.assertValid("items.find({\"repo\": \"works\"}).include(\"repo\", \"property.*\").transitive()");
        this.assertValid("items.find({\"repo\": \"works\"}).include(\"repo\", \"property.key\").transitive()");
        this.assertValid("items.find({\"repo\": \"works\"}).include(\"repo\", \"property.value\").transitive()");
        this.assertValid("items.find({\"repo\": \"works\"}).include(\"repo\", \"@bla\").transitive()");
        this.assertInvalid("items.find({\"repo\": \"works\"}).include(\"repo\", \"@bla\").transitive().offset(10)");
        this.assertInvalid("items.find({\"repo\": \"works\"}).include(\"repo\", \"@bla\").transitive().sort({\"$asc" +
                "\" : [\"repo\"]})");
        this.assertInvalid("artifacts.find().transitive()");
        this.assertInvalid("items.find().include(\"repo\", \"artifact.module.build.name\").transitive()");
        this.assertInvalid("items.find().include(\"repo\", \"artifact.module.build.@bla\").transitive()");
        this.assertInvalid(
                "items.find({\"repo\": \"works\"}).include(\"repo\", \"artifact.module.build.name\").transitive()");
        this.assertInvalid(
                "items.find({\"name\": \"worksnot\"}).transitive()");
        this.assertInvalid(
                "items.find({\"name\": \"worksnot\"}).include(\"repo\").transitive()");
        this.assertInvalid(
                "items.find({\"name\": \"worksnot\"}).include(\"repo\", \"@bla\").transitive()");
        this.assertInvalid(
                "items.find({\"$or\":[{\"repo\": \"worksnot\"},{\"repo\": \"worksnot2\"}]}).transitive()");
        this.assertInvalid("items.find({\"repo\": \"worksnot\"},{\"repo\": \"worksnot2\"}).transitive()");
        this.assertInvalid(
                "items.find({\"$or\":[{\"repo\": \"worksnot\"},{\"repo\": \"worksnot2\"}]}).include(\"repo\", " +
                        "\"@bla\").transitive()");
    }

    private void assertValid(String query) {
        AqlPermissionProvider permissionMock = Mockito.mock(AqlPermissionProvider.class);
        when(permissionMock.isOss()).thenReturn(false);
        when(permissionMock.isAdmin()).thenReturn(true);
        ParserElementResultContainer container = this.parser.parse(query);
        AqlQuery aqlQuery = this.aqlAdapter.toAqlModel(container);
        this.validator.validate(aqlQuery, permissionMock);
    }

    private void assertInvalid(String query) {
        try {
            AqlPermissionProvider permissionMock = Mockito.mock(AqlPermissionProvider.class);
            when(permissionMock.isOss()).thenReturn(false);
            when(permissionMock.isAdmin()).thenReturn(true);
            ParserElementResultContainer container = this.parser.parse(query);
            AqlQuery aqlQuery = this.aqlAdapter.toAqlModel(container);
            this.validator.validate(aqlQuery, permissionMock);
            Assert.fail();
        } catch (Exception e) {
            Assert.assertTrue(true);
        }
    }
}
